<?php
/*
* Woocommerce eShipper shipping related works are here
*/
if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins'))))
{
	function eShipper_init()
	{
		if (!class_exists('WC_eShipper'))
		{
			class WC_eShipper extends WC_Shipping_Method
			{
				/**
				 * Constructor for your shipping class
				 *
				 * @access public
				 * @return void
				 */
				public function __construct()
				{
					$this->id = 'WC_eShipper'; // Id for your shipping method. Should be uunique.
					$this->method_title = __('eShipper', 'WC_eShipper'); // Title shown in admin
					$this->method_description = __('eShipper', 'WC_eShipper'); // Description shown in admin
					$this->init();
				}

				/**
				 * Init function
				 *
				 * @access public
				 * @return void
				 */
				function init()
				{
					$default_options = (object)array(
						'enabled' => 'no',
						'courier' => 'yes',
						'ltl' => 'yes',
						'residential' => 'no',
						'signature' => 'no',
						'cheapest' => 'no',
						'boxing_enabled' => 'no',
						'title' => 'eShipper',
						'api_url' => 'https://web.eshipper.com/rpc2',
						'api_user' => '',
						'api_pass' => '',
						'source_postalcode' => '',
						'source_country' => '',
						'source_province' => '',
						'source_city' => '',
						'source_name' => '',
						'source_address1' => '',
						'source_address2' => '',
						'source_address3' => '',
						'source_phone' => '',
						'source_fax' => '',
						'source_address_type' => '',
						'source_company_name' => '',
						'enable_4D_boxing' => 'no',		//K4D
						//K4D 'user_api_key' => '',
						'boxes' => '',
						'increase_rate' => 0
					);
					$this->options = get_option('woocommerce_eShipper', $default_options);
					$this->enabled = $this->options->enabled;
					$this->title = $this->options->title;
					//K4D update_option('user_api_key','');
					// Save settings in admin if you have any defined
					add_action('woocommerce_update_options_shipping_' . $this->id, array(
						$this,
						'process_admin_options'
					));

				}

				/**
				 * calculate_shipping function.
				 *
				 * @access public
				 * @param mixed $package
				 * @return void
				*/


				public function calculate_shipping($packages = array())
				{                   
					$this->shipping_total = 0;
					$this->shipping_taxes = array();
					$this->packages = array();
					if (!function_exists('xml2js'))
					{
						function xml2js($xmlnode)
						{
							$root = (func_num_args() > 1 ? false : true);
							$jsnode = array();
							if (!$root)
							{
								if (count($xmlnode->attributes()) > 0)
								{
									$jsnode["$"] = array();
									foreach($xmlnode->attributes() as $key => $value) 
										$jsnode["$"][$key] = (string)$value;
								}
								$textcontent = trim((string)$xmlnode);
								if (count($textcontent) > 0)
									$jsnode["<br />"] = $textcontent;
								foreach($xmlnode->children() as $childxmlnode)
								{
									$childname = $childxmlnode->getName();
									if (!array_key_exists($childname, $jsnode))
										$jsnode[$childname] = array();
									array_push($jsnode[$childname], xml2js($childxmlnode, true));
								}
								return $jsnode;
							}
							else
							{
								$nodename = $xmlnode->getName();
								$jsnode[$nodename] = array();
								array_push($jsnode[$nodename], xml2js($xmlnode, true));
								return json_encode($jsnode);
							}
						}
					}

					global $woocommerce;

					// currency
					$eShipper_currency = get_option('woocommerce_currency');
					$woocommerce_eShipper_settings = (array)get_option("woocommerce_eShipper");	 // origin details
                
					$eShipper_origin_country = $woocommerce_eShipper_settings['source_country'];
					$eShipper_origin_postal_code = $woocommerce_eShipper_settings['source_postalcode'];
					$eShipper_origin_province = $woocommerce_eShipper_settings['source_province'];
					$eShipper_origin_city = $woocommerce_eShipper_settings['source_city'];
					$eShipper_origin_name = $woocommerce_eShipper_settings['source_name'];
					$eShipper_origin_address1 = $woocommerce_eShipper_settings['source_address1'];
					$eShipper_origin_address2 = $woocommerce_eShipper_settings['source_address2'];
					$eShipper_origin_address3 = $woocommerce_eShipper_settings['source_address3'];
					$eShipper_origin_phone = $woocommerce_eShipper_settings['source_phone'];
					$eShipper_origin_fax = $woocommerce_eShipper_settings['source_fax'];
					$eShipper_origin_address_type = $woocommerce_eShipper_settings['source_address_type'];
					$eShipper_origin_company_name = $woocommerce_eShipper_settings['source_company_name'];

					// api settings

					$eShipper_ltl_rates = $woocommerce_eShipper_settings['ltl'];
					$eShipper_courier_rates = $woocommerce_eShipper_settings['courier'];
          			$girthflag = 0;
          			$packagegirth = 0;
          			if($eShipper_ltl_rates == 'yes' && $eShipper_courier_rates == 'yes')
					{
          				$packagetype = 'Package';
          				$girthflag = 1;
          			}
          			elseif($eShipper_ltl_rates == 'yes' && $eShipper_courier_rates == 'no')
					{
          				$packagetype = 'Pallet';
          			}
          			elseif($eShipper_ltl_rates == 'no' && $eShipper_courier_rates == 'yes')
					{
          				$packagetype = 'Package';
          			}
          			elseif($eShipper_ltl_rates == 'no' && $eShipper_courier_rates == 'no')
					{
          				return;
          			}

					//K4D
					$boxing_enabled = 0;
                    //if(($woocommerce_eShipper_settings['enable_4D_boxing'] == 'yes') && (!empty($woocommerce_eShipper_settings['boxes'])))
					$boxing_attr = $woocommerce_eShipper_settings['boxing_enabled'];
					if($boxing_attr == 'yes')
					{
						$boxing_enabled = 1;
					}

					$api_username = $woocommerce_eShipper_settings['api_user'];
					$api_password = $woocommerce_eShipper_settings['api_pass'];
					$api_url = $woocommerce_eShipper_settings['api_url'];
					$user_api_key = $woocommerce_eShipper_settings['user_api_key'];
					
					// destination details

					$user_id = get_current_user_id();
					$woocusobj = $woocommerce->customer;
					$eShipper_destintn_country = $woocusobj->get_shipping_country();
					$eShipper_destintn_postal_code = $woocusobj->get_shipping_postcode();
					$eShipper_destintn_province = $woocusobj->get_shipping_state();
					$eShipper_destintn_city = $woocusobj->get_shipping_city();
					$eShipper_destintn_name = get_user_meta($user_id, 'shipping_first_name', true);
					$eShipper_destintn_address1 = $woocusobj->get_shipping_address();
					$eShipper_destintn_address2 = $woocusobj->get_shipping_address_2();
					$eShipper_destintn_address3 = "";
					$eShipper_destintn_phone = "";
					$eShipper_destintn_fax = "";
					$eShipper_destintn_address_type = "";
					$eShipper_destintn_company_name = get_user_meta($user_id, 'shipping_company', true);
					$item_string = "";
					$total_item_weight = 0;
					$items = $woocommerce->cart->get_cart();
                    
					// xml_data

					$xml = new SimpleXMLElement('<EShipper></EShipper>');
					$xml->addAttribute('xmlns', $api_url);
					$xml->addAttribute('username', $api_username);
					$xml->addAttribute('password', $api_password);
					$xml->addAttribute('version', '3.0.0');
					$qrequest = $xml->addChild('QuoteRequest');
					$qrequest->addAttribute('serviceId', '0');
					$qrequest->addAttribute('stackable', 'true');
					$from = $qrequest->addChild("From");
					$from->addAttribute('id', '1');
					if(($eShipper_origin_name == "") || ($eShipper_origin_name == " "))
					{
						$eShipper_origin_name = "ABC";
					}
					$from->addAttribute('company', $eShipper_origin_name);
					if(($eShipper_origin_address1 == "") || ($eShipper_origin_address1 == " "))
					{
						$eShipper_origin_address1 = "123-ABC";
					}
					$from->addAttribute('address1', $eShipper_origin_address1);
					$from->addAttribute('city', $eShipper_origin_city);
					$from->addAttribute('state', $eShipper_origin_province);
					$from->addAttribute('country', $eShipper_origin_country);
					$from->addAttribute('zip', $eShipper_origin_postal_code);
					$to = $qrequest->addChild("To");
					if(($eShipper_destintn_company_name == "") || ($eShipper_destintn_company_name == " "))
					{
						$eShipper_destintn_company_name = "ABC";
					}
					$to->addAttribute('company', $eShipper_destintn_company_name);
					if(($eShipper_destintn_address1 == "") || ($eShipper_destintn_address1 == " "))
					{
						$eShipper_destintn_address1 = "123-ABC";
					}
					$to->addAttribute('address1', $eShipper_destintn_address1);
					$to->addAttribute('city', $eShipper_destintn_city);
					$to->addAttribute('state', $eShipper_destintn_province);
					$to->addAttribute('zip', $eShipper_destintn_postal_code);
					$to->addAttribute('country', $eShipper_destintn_country);

					$residentialattr  = $woocommerce_eShipper_settings['residential'];
					if($residentialattr == 'yes')
					{
						$to->addAttribute('residential', 'true');

						if($woocommerce_eShipper_settings['ltl'])
						{
							$to->addAttribute('tailgateRequired', 'true');
						}
					}
					else
					{
						$to->addAttribute('residential', 'false');
					}	
					
					$signatureattr  = $woocommerce_eShipper_settings['signature'];
					if($signatureattr == 'yes')
					{
						$qrequest->addAttribute('signatureRequired', 'Yes');
					}
					else
					{
						$to->addAttribute('signatureRequired', 'No');
					}	

					/*Kanika - Removing COD
					$cod = $qrequest->addChild("COD");
					$cod->addAttribute('paymentType', 'None');
					$codr = $cod->addChild("CODReturnAddress");
					$codr->addAttribute('codCompany', $eShipper_destintn_company_name);
					$codr->addAttribute('codName', $eShipper_destintn_name);
					$codr->addAttribute('codPhone', '888-888-8888');
					$codr->addAttribute('codAddress1', $eShipper_destintn_address1);
					$codr->addAttribute('codAddress2', $eShipper_destintn_address1);
					$codr->addAttribute('codCity', $eShipper_destintn_city);
					$codr->addAttribute('codStateCode', $eShipper_destintn_province);
					$codr->addAttribute('codZip', $eShipper_destintn_postal_code);
					$codr->addAttribute('codCountry', 'CA');
					*/
					
					$package = $qrequest->addChild("Packages");

					if(($eShipper_destintn_postal_code == "") || ($eShipper_destintn_postal_code == " ") || ($eShipper_origin_postal_code == "") || ($eShipper_origin_postal_code == " "))
					{
						return;
					}
					if (empty($packages))
					{
						return;
					}
					// $packages as $package_key => $package
					$itemcount = 0;

					/* K4D
					// 4dbox api request array
					if(!empty($user_api_key)){
						$request = array('username'	=> $api_username,'api_key'	=> $user_api_key,'units' => 'in','onebin'	=> true,);
					}
					else{
						$request = array();
					}
					*/
					
					//K4D
					if($boxing_enabled == 1)
					{
						$boxing_xml = new SimpleXMLElement('<EShipper></EShipper>');
						$boxing_xml->addAttribute('xmlns', $api_url);
						$boxing_xml->addAttribute('username', $api_username);
						$boxing_xml->addAttribute('password', $api_password);
						$boxing_xml->addAttribute('version', '3.0.0');
						$box_request = $boxing_xml->addChild('BoxingRequest');
						$items_arr = $box_request->addChild('Items');
						$boxes_arr = $box_request->addChild('Boxes');
												
						$boxesaary = $woocommerce_eShipper_settings['boxes'];
                        $box_no = 1;
                        foreach ($boxesaary as $key => $boxvalue) 
						{
							$box = $boxes_arr->addChild('Box');
							$box->addAttribute('name', 'Box '.$box_no);		// Box Name (string)
							$box->addAttribute('length', $boxvalue[2]);		// Box Length (float)
							$box->addAttribute('width', $boxvalue[1]);		// Box Width (float)
							$box->addAttribute('height', $boxvalue[0]);		// Box Height (float)
							$box->addAttribute('max', $boxvalue[4]);		// Maximum Weight (float)
							$box->addAttribute('weight', $boxvalue[3]);		// Box Weight - Accommodate Packing Material Weight (float)
							$box_no++;
                        }
					}
					
					$packageitemsarray = array();
					foreach($packages['contents'] as $item_id => $values)
					{
						// $_product =  wc_get_product( $values['data']->get_id());
						$_product = $values['data'];
						$productmeta = new WC_Product($values['product_id']);
						$item_name = $_product->get_name();
						$item_sku = $_product->get_sku();
						$item_quantity = $values['quantity'];
						$itemcount = $itemcount + $item_quantity;

						/*==============================convert width,height, length and weight into in and lbs==============================*/
						$item_width1 = $_product->get_width();
						$item_height1 = $_product->get_height();
						$item_length1 = $_product->get_length();
						$item_weight1 = $_product->get_weight();
					    
					    $weight_unit = get_option('woocommerce_weight_unit');
                        $dimension_unit = get_option('woocommerce_dimension_unit');

						$item_width = wc_get_dimension( $item_width1, 'in', $dimension_unit );
						$item_height = wc_get_dimension( $item_height1, 'in', $dimension_unit );
						$item_length = wc_get_dimension( $item_length1, 'in', $dimension_unit );
						$item_weight = wc_get_weight($item_weight1, 'lbs', $weight_unit );
                     
						/*============================convert width,height, length and weight into in and lbs End==========================*/          

						$request['items'][] = array(
							'name'		=> $item_name,					// Product Name (string)
							'quantity'	=> $item_quantity,				// Product Quantity (int)
							'length'	=> $item_length,				// Product Length (float)
							'width'		=> $item_width,					// Product Width (float)
							'height'	=> $item_height,				// Product Height (float)
							'weight'	=> $item_weight					// Product Weight (float)
						);	
						
						//K4D
						if($boxing_enabled == 1)
						{
							$item = $items_arr->addChild('Item');
							$item->addAttribute('name', $item_name);			// Product Name (string)
							$item->addAttribute('quantity', $item_quantity);	// Product Quantity (int)
							if($item_length <= 0)
							{
								$item->addAttribute('length', '1');
							}
							else
							{
								$item->addAttribute('length', $item_length);		// Product Length (float)
							}
							if($item_width <= 0)
							{
								$item->addAttribute('width', '1');
							}
							else
							{
								$item->addAttribute('width', $item_width);			// Product Width (float)
							}
							if($item_height <= 0)
							{
								$item->addAttribute('height', '1');
							}
							else
							{
								$item->addAttribute('height', $item_height);		// Product Height (float)
							}
							if($item_weight <= 0)
							{
								$item->addAttribute('weight', '1');
							}
							else
							{
								$item->addAttribute('weight', $item_weight);		// Product Weight (float)
							}
						}

						$total_item_weight += (int)($item_weight * $item_quantity);
						$total_item_height = (int)total_item_height + ((int)$item_height * (int)$item_quantity); // Considering items with same dimensions get stacked one upon another.
						$item_weightUnit = get_option('woocommerce_weight_unit');
						$item_price = $_product->get_price();
						$item_vendor = "rizeShipper";
						$item_requires_shipping = 'true';
						$item_taxable = 'true';
						$item_fulfillment_service = "manual";
						$item_product_id = $values['product_id'];
						$item_variant_id = $values['variation_id'];

						if ($item_sku == '')
						{
							$item_sku = '12345';
						}

						if (($item_weight == '') || ($item_weight == '0'))
						{
							$item_weight = '1';
						}

						if ($item_variant_id == '')
						{
							$item_variant_id = '616173013';
						}

						$packageitemsarray[] = array('length'=>$item_length,'width'=>$item_width,'height'=>$total_item_height,'weight'=>$total_item_weight,'desc'=>$item_name);	

					}  // end foreach

					// continue xml data

					//K4D
					$boxing_success = 1; //Flag to monitor 4D-Boxing Response
					if(($itemcount > 1) && ($boxing_enabled == 1))
					{
						$boxing_success = 0;	//Flag to monitor 4D-Boxing Response
						$boxing_xmldata = $boxing_xml->asXML();
						$url = $api_url;
						$ch = curl_init();
						curl_setopt($ch, CURLOPT_URL, $url);
						curl_setopt($ch, CURLOPT_POST, true);
						curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: appplication/xml'));
						curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
						curl_setopt($ch, CURLOPT_POSTFIELDS, $boxing_xmldata);
						$boxing_result = curl_exec($ch);
						$errno 		= curl_errno($ch);
						$error 		= curl_error($ch);
						curl_close($ch);
						
						$boxing_response_xml = simplexml_load_string($boxing_result);	//Loads boxing_result as a string object in - boxing_response_xml
						
						$boxing_xml = $boxing_response_xml;
						if(!empty($boxing_xml->BoxingRequestReply->PackedBoxes->Box))
						{
						foreach ($boxing_xml->BoxingRequestReply->PackedBoxes->Box as $box)
						{
							$boxing_success = 1;
						
							/* Code to Update Packages for Quote-Request Start*/
							
							//Sort Dimensions - To Be Added
									
							//Determine Package Type
							if($girthflag)
							{
								$itemgirth = ($box['width'] + $box['height']) * 2;
								$item_dimension = $box['length'] + $itemgirth;
								if($box['length'] < 108 && $box['weight'] < 150 && $item_dimension < 165){
									$service_type = "Package";
								}
								else
								{
									$service_type = "Pallet";
								}
							}
							else
							{
								$service_type = "Package";
							}
							
							if ($packagetype == "Pallet")
							{
								//Final Package Type will be Pallet only.
							}
							else
							{
								$packagetype = $service_type;
							}
	
							$box['width']  = round($box['width'],0,PHP_ROUND_HALF_UP);
							$box['height'] = round($box['height'],0,PHP_ROUND_HALF_UP);
							$box['length'] = round($box['length'],0,PHP_ROUND_HALF_UP);
							$box['weight'] = round($box['weight'],0,PHP_ROUND_HALF_UP);
								
							if ($box['weight'] <= 0)
							{
								$box['weight'] = 1;
							}
							if ($box['width'] <= 0)
							{
								$box['width'] = 1;
							}
							if ($box['height'] <= 0)
							{
								$box['height'] = 1;
							}
							if ($box['length'] <= 0)
							{
								$box['length'] = 1;
							}

							$packages = $package->addChild("Package");
							$packages->addAttribute('length', $box['length']);
							$packages->addAttribute('width', $box['width']);
							$packages->addAttribute('height', $box['height']);
							$packages->addAttribute('weight', $box['weight']);
							$packages->addAttribute('type', $service_type);
							$packages->addAttribute('freightClass', '70');
							$packages->addAttribute('nmfcCode', '123456');
							$packages->addAttribute('insuranceAmount', '0.0');
							$packages->addAttribute('codAmount', '0.0');
							$packages->addAttribute('description', $box['name']);
						
							/* Code to Update Packages for Quote-Request End*/
						}
						}
						else
						{
							//BOX Array is EMPTY
						}
						/* Code to Update Custome Items in Package List */
						if(!empty($boxing_xml->BoxingRequestReply->CustomItems))
						{
						foreach ($boxing_xml->BoxingRequestReply->CustomItems->Item as $customItem)
						{
							$boxing_success = 1;

							/* Code to Update Packages for Quote-Request Start*/
							
							//Sort Dimensions - To Be Added

							//Determine Package Type
							if($girthflag)
							{
								$itemgirth = ($customItem['width'] + $customItem['height']) * 2;
								$item_dimension = $customItem['length'] + $itemgirth;
								if($packedBox['length'] < 108 && $customItem['weight'] < 150 && $item_dimension < 165){
									$service_type = "Package";
								}
								else
								{
									$service_type = "Pallet";
								}
							}
							else
							{
								$service_type = "Package";
							}
							
							if ($packagetype == "Pallet")
							{
								//Final Package Type will be Pallet only.
							}
							else
							{
								$packagetype = $service_type;
							}
							
							$customItem['width']  = round($customItem['width'],0,PHP_ROUND_HALF_UP);
							$customItem['height'] = round($customItem['height'],0,PHP_ROUND_HALF_UP);
							$customItem['length'] = round($customItem['length'],0,PHP_ROUND_HALF_UP);
							$customItem['weight'] = round($customItem['weight'],0,PHP_ROUND_HALF_UP);
							if ($customItem['weight'] <= 0)
							{
								$customItem['weight'] = 1;
							}
							if ($customItem['width'] <= 0)
							{
								$customItem['width'] = 1;
							}
							if ($customItem['height'] <= 0)
							{
								$customItem['height'] = 1;
							}
							if ($customItem['length'] <= 0)
							{
								$customItem['length'] = 1;
							}
							
							for($i=0;$i<$customItem['quantity'];$i++)
							{
								$packages = $packages->addChild("Package");
								$packages->addAttribute('length', $customItem['length']);
								$packages->addAttribute('width', $customItem['width']);
								$packages->addAttribute('height', $customItem['height']);
								$packages->addAttribute('weight', $customItem['weight']);
								$packages->addAttribute('type', $service_type);
								$packages->addAttribute('freightClass', '70');
								$packages->addAttribute('nmfcCode', '123456');
								$packages->addAttribute('insuranceAmount', '0.0');
								$packages->addAttribute('codAmount', '0.0');
								$packages->addAttribute('description', $customItem['name']);
							}
							/* Code to Update Packages for Quote-Request End*/
						}
						}
						else
						{
							//No Custom Items
						}
						}
					/* K4D Commenting 4D Boxing Error Scenario  --> Implemented with Check: ($boxing_success==0)
						 else {
							// 4d box error
							if(!empty($woocommerce_eShipper_settings['boxes'])){
                               $boxesaary = $woocommerce_eShipper_settings['boxes'];
                               if(count($boxesaary) > 1){
                               	$boxindex = array_search(min($boxesaary), $boxesaary);
                               	$boxweight = $boxesaary[$boxindex][3]+$boxesaary[$boxindex][4];
								
								// Determine Length: Measure the longest side of the packed Box
								$calc_length = (($boxesaary[$boxindex][2] > $boxesaary[$boxindex][1]) ? (($boxesaary[$boxindex][2] > $boxesaary[$boxindex][0]) ? ($boxesaary[$boxindex][2]) : ($boxesaary[$boxindex][0])) : (($boxesaary[$boxindex][1] > $boxesaary[$boxindex][0]) ? ($boxesaary[$boxindex][1]) : ($boxesaary[$boxindex][0])));
								if ($calc_length == $packedBox['length'])
								{
									//Do Nothing
								}
								else if ($calc_length == $boxesaary[$boxindex][1])
								{
									$boxesaary[$boxindex][1] = $boxesaary[$boxindex][2];
									$boxesaary[$boxindex][2] = $calc_length;
								}
								else if ($calc_length == $boxesaary[$boxindex][0])
								{
									$boxesaary[$boxindex][0] = $boxesaary[$boxindex][2];
									$boxesaary[$boxindex][2] = $calc_length;
								}
								$itemgirth = ($boxesaary[$boxindex][1]+$boxesaary[$boxindex][0])*2;
								$item_dimension = $boxesaary[$boxindex][2] + $itemgirth;
								if($packedBox['weight'] >= 150 || $item_dimension >= 165 || $boxesaary[$boxindex][2] > 108){
									$packagegirth = 1;
								}

                               	if($packagegirth == 1 && $girthflag == 1){
									$packagetype = 'Pallet';
								}
								
								$boxesaary[$boxindex][1]  = round($boxesaary[$boxindex][1],0,PHP_ROUND_HALF_UP);
								$boxesaary[$boxindex][0] = round($boxesaary[$boxindex][0],0,PHP_ROUND_HALF_UP);
								$boxesaary[$boxindex][2] = round($boxesaary[$boxindex][2],0,PHP_ROUND_HALF_UP);
								$boxweight = round($boxweight,0,PHP_ROUND_HALF_UP);
								if ($boxweight < 1)
								{
									$boxweight = 1;
								}
								
								$packages = $package->addChild("Package");
								$packages->addAttribute('length', $boxesaary[$boxindex][2]);
								$packages->addAttribute('width', $boxesaary[$boxindex][1]);
								$packages->addAttribute('height', $boxesaary[$boxindex][0]);
								$packages->addAttribute('weight', $boxweight);
								$packages->addAttribute('type', $packagetype);
								$packages->addAttribute('freightClass', '70');
								$packages->addAttribute('nmfcCode', '123456');
								$packages->addAttribute('insuranceAmount', '0.0');
								$packages->addAttribute('codAmount', '0.0');
								$packages->addAttribute('description', 'Box new');
                               }else{
                               
                         foreach ($boxesaary as $key => $boxvalue) {    
								$boxweight = $boxvalue[3]+$boxvalue[4];
								
								// Determine Length: Measure the longest side of the packed Box
								$calc_length = (($boxvalue[2] > $boxvalue[1]) ? (($boxvalue[2] > $boxvalue[0]) ? ($boxvalue[2]) : ($boxvalue[0])) : (($boxvalue[1] > $boxvalue[0]) ? ($boxvalue[1]) : ($boxvalue[0])));
								if ($calc_length == $boxvalue[2])
								{
									//Do Nothing
								}
								else if ($calc_length == $boxvalue[1])
								{
									$boxvalue[1] = $boxvalue[2];
									$boxvalue[2] = $calc_length;
								}
								else if ($calc_length == $boxvalue[0])
								{
									$boxvalue[0] = $boxvalue[2];
									$boxvalue[2] = $calc_length;
								}
								$itemgirth = ($boxvalue[1]+$boxvalue[0])*2;
								$item_dimension = $boxvalue[2] + $itemgirth;
								if($packedBox['weight'] >= 150 || $item_dimension >= 165 || $item_length > 108){
									$packagegirth = 1;
								}

                               	if($packagegirth == 1 && $girthflag == 1){
									$packagetype = 'Pallet';
								}
								
								$boxvalue[1]  = round($boxvalue[1],0,PHP_ROUND_HALF_UP);
								$boxvalue[0] = round($boxvalue[0],0,PHP_ROUND_HALF_UP);
								$boxvalue[2] = round($boxvalue[2],0,PHP_ROUND_HALF_UP);
								$boxweight = round($boxweight,0,PHP_ROUND_HALF_UP);
								if ($boxweight < 1)
								{
									$boxweight = 1;
								}
								
								$packages = $package->addChild("Package");
								$packages->addAttribute('length', $boxvalue[2]);
								$packages->addAttribute('width', $boxvalue[1]);
								$packages->addAttribute('height', $boxvalue[0]);
								$packages->addAttribute('weight', $boxweight);
								$packages->addAttribute('type', $packagetype);
								$packages->addAttribute('freightClass', '70');
								$packages->addAttribute('nmfcCode', '123456');
								$packages->addAttribute('insuranceAmount', '0.0');
								$packages->addAttribute('codAmount', '0.0');
								$packages->addAttribute('description', 'Box single 4d');
										}
                               }
                           }

						} // else
					

						} K4D */
					else	// If Item Count <= 1 OR 4D-Boxing is Not Enabled
					{
						if(!empty($packageitemsarray))
						{
							foreach ($packageitemsarray as $packitemvalue) 
							{
								//Kanika: Fix for issue: "Package Girth was not getting calculated if Quantity=1"	25 Jan 2019 
								// calculate packagegirth
								// Determine Length: Measure the longest side of the package
								$calc_length = (($packitemvalue['length'] > $packitemvalue['width']) ? (($packitemvalue['length'] > $packitemvalue['height']) ? ($packitemvalue['length']) : ($packitemvalue['height'])) : (($packitemvalue['width'] > $packitemvalue['height']) ? ($packitemvalue['width']) : ($packitemvalue['height'])));
								if ($calc_length == $packitemvalue['length'])
								{
									//Do Nothing
								}
								else if ($calc_length == $packitemvalue['width'])
								{
									$packitemvalue['width'] = $packitemvalue['length'];
									$packitemvalue['length'] = $calc_length;
								}
								else if ($calc_length == $packitemvalue['height'])
								{
									$packitemvalue['height'] = $packitemvalue['length'];
									$packitemvalue['length'] = $calc_length;
								}
								
								$itemgirth = ((int)$packitemvalue['width'] + (int)$packitemvalue['height'])*2;
								$item_dimension = (int)$packitemvalue['length'] + (int)$itemgirth;
								if($total_item_weight >= 150 || $item_dimension >= 165 || $packitemvalue['length'] > 108)
								{
									$packagegirth = 1;
								}
								if($packagegirth == 1 && $girthflag == 1)
								{
									$packagetype = 'Pallet';
								}
						
								$packitemvalue['width']  = round($packitemvalue['width'],0,PHP_ROUND_HALF_UP);
								$packitemvalue['height'] = round($packitemvalue['height'],0,PHP_ROUND_HALF_UP);
								$packitemvalue['length'] = round($packitemvalue['length'],0,PHP_ROUND_HALF_UP);
								$packitemvalue['weight'] = round($packitemvalue['weight'],0,PHP_ROUND_HALF_UP);
								if ($packitemvalue['weight'] <= 0)
								{
									$packitemvalue['weight'] = 1;
								}
								if ($packitemvalue['width'] <= 0)
								{
									$packitemvalue['width'] = 1;
								}
								if ($packitemvalue['length'] <= 0)
								{
									$packitemvalue['length'] = 1;
								}
								if ($packitemvalue['height'] <= 0)
								{
									$packitemvalue['height'] = 1;
								}
								
								$packages = $package->addChild("Package");
								$packages->addAttribute('length', $packitemvalue['length']);
								$packages->addAttribute('width', $packitemvalue['width']);
								$packages->addAttribute('height', $packitemvalue['height']);
								$packages->addAttribute('weight', $packitemvalue['weight']);
								$packages->addAttribute('type', $packagetype);
								$packages->addAttribute('freightClass', '70');
								$packages->addAttribute('nmfcCode', '123456');
								$packages->addAttribute('insuranceAmount', '0.0');
								$packages->addAttribute('codAmount', '0.0');
								$packages->addAttribute('description', $packitemvalue['desc']);

							}
						}

					}
					
										
					if ($boxing_success == 0)
					{
						$ch = curl_init();
						curl_setopt($ch, CURLOPT_URL, $url);
						curl_setopt($ch, CURLOPT_POST, true);
						curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: appplication/xml'));
						curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
						curl_setopt($ch, CURLOPT_POSTFIELDS, "KANIKA - 6: Boxing Success=0");
						$temp_result = curl_exec($ch);
						curl_close($ch);
						
						
						// Implement Generic Functionality (If Item Count <= 1 OR 4D-Boxing is Not Enabled) if 4D-Boxing returned Neither Boxes Nor Custom Items
						if(!empty($packageitemsarray))
						{
							foreach ($packageitemsarray as $packitemvalue) 
							{
								//Kanika: Fix for issue: "Package Girth was not getting calculated if Quantity=1"	25 Jan 2019 
								// calculate packagegirth
								// Determine Length: Measure the longest side of the package
								$calc_length = (($packitemvalue['length'] > $packitemvalue['width']) ? (($packitemvalue['length'] > $packitemvalue['height']) ? ($packitemvalue['length']) : ($packitemvalue['height'])) : (($packitemvalue['width'] > $packitemvalue['height']) ? ($packitemvalue['width']) : ($packitemvalue['height'])));
								if ($calc_length == $packitemvalue['length'])
								{
									//Do Nothing
								}
								else if ($calc_length == $packitemvalue['width'])
								{
									$packitemvalue['width'] = $packitemvalue['length'];
									$packitemvalue['length'] = $calc_length;
								}
								else if ($calc_length == $packitemvalue['height'])
								{
									$packitemvalue['height'] = $packitemvalue['length'];
									$packitemvalue['length'] = $calc_length;
								}
								
								$itemgirth = ($packitemvalue['width']+$packitemvalue['height'])*2;
								$item_dimension = $packitemvalue['length'] + $itemgirth;
								if($total_item_weight >= 150 || $item_dimension >= 165 || $packitemvalue['length'] > 108)
								{
									$packagegirth = 1;
								}
								if($packagegirth == 1 && $girthflag == 1)
								{
									$packagetype = 'Pallet';
								}
						
								$packitemvalue['width']  = round($packitemvalue['width'],0,PHP_ROUND_HALF_UP);
								$packitemvalue['height'] = round($packitemvalue['height'],0,PHP_ROUND_HALF_UP);
								$packitemvalue['length'] = round($packitemvalue['length'],0,PHP_ROUND_HALF_UP);
								$packitemvalue['weight'] = round($packitemvalue['weight'],0,PHP_ROUND_HALF_UP);
								if ($packitemvalue['weight'] <= 0)
								{
									$packitemvalue['weight'] = 1;
								}
								if ($packitemvalue['width'] <= 0)
								{
									$packitemvalue['width'] = 1;
								}
								if ($packitemvalue['length'] <= 0)
								{
									$packitemvalue['length'] = 1;
								}
								if ($packitemvalue['height'] <= 0)
								{
									$packitemvalue['height'] = 1;
								}
								
								$packages = $package->addChild("Package");
								$packages->addAttribute('length', $packitemvalue['length']);
								$packages->addAttribute('width', $packitemvalue['width']);
								$packages->addAttribute('height', $packitemvalue['height']);
								$packages->addAttribute('weight', $packitemvalue['weight']);
								$packages->addAttribute('type', $packagetype);
								$packages->addAttribute('freightClass', '70');
								$packages->addAttribute('nmfcCode', '123456');
								$packages->addAttribute('insuranceAmount', '0.0');
								$packages->addAttribute('codAmount', '0.0');
								$packages->addAttribute('description', $packitemvalue['desc']);

							}
						}
					}
					/* Commenting PICKUP
					$pickup = $qrequest->addChild("Pickup");
					$pickup->addAttribute('contactName', $eShipper_origin_name);
					$pickup->addAttribute('phoneNumber', '888-888-8888');
					$pickup->addAttribute('pickupDate', '2009-08-03');
					$pickup->addAttribute('pickupTime', '16:30');
					$pickup->addAttribute('closingTime', '17:45');
					$pickup->addAttribute('location', 'Front Door');
					*/

					$package->addAttribute('type', $packagetype);

					// endxml

					$xmldata = $xml->asXML();

					//echo 'xml:'.$xmldata;
					//die();

					$url = $api_url;
					$increase_rate = $woocommerce_eShipper_settings['increase_rate'];
					$increase_rate = (float)$increase_rate;
					$ch = curl_init();
					curl_setopt($ch, CURLOPT_URL, $url);
					curl_setopt($ch, CURLOPT_POST, true);
					curl_setopt($ch, CURLOPT_HTTPHEADER, array(
						'Content-Type: appplication/xml'
					));
					curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
					curl_setopt($ch, CURLOPT_POSTFIELDS, $xmldata);
					$result = curl_exec($ch);
					// echo curl_error($ch);
					curl_close($ch);
					if (!$result || strlen(trim($result)) == 2)
					{
						$rate = array();
						$this->add_rate($rate);
					}
					else
					{
						$xml = simplexml_load_string($result);

						$jsondata  = json_encode($xml);
 						$resultdata = json_decode($jsondata, true);
 						$base_currency = get_woocommerce_currency();
 						//echo '<pre>';
 						//print_r($resultdata);
 						//die();
 						if(isset($resultdata['QuoteReply']) && !empty($resultdata['QuoteReply']['Quote'])) 
						{
 							$qutoereplyarray = $resultdata['QuoteReply']['Quote'];

							//Code Changes to show Cheapest Option Only
							//Assumption: eShipper sends Quote Response in increasing order of Price. So we just need to display the first option received.
							$count = 0;
							$cheapestOnly  = $woocommerce_eShipper_settings['cheapest'];
					
							foreach($qutoereplyarray as $resultshipping)
							{
								$count++;
								if($cheapestOnly == 'yes' && $count > 1)
								{
									break;
								}
								$max_delivery_date = "";
								if (!empty($resultshipping['@attributes']['deliveryDate']))
								{
									$expected_day = $resultshipping['@attributes']['transitDays'] + 2;
									$max_delivery_date = " (Estimated Delivery : " . $resultshipping['@attributes']['transitDays'] . "-" . $expected_day . " days) ";
								}
								else
								{
									$max_delivery_date = "";
								}
								$price = $resultshipping['@attributes']['totalCharge'];
								$np_label_shipping = $resultshipping['@attributes']['carrierName'] ." - ". $resultshipping['@attributes']['serviceName'] . $max_delivery_date;
								$shipingPrice=($price + ($price * $increase_rate * 0.01));
								if($base_currency != 'CAD' && $resultshipping['@attributes']['currency'] == 'CAD')
								{
									$finbasecurrency = 'CAD'.$base_currency;
									$finalconprice=$this->currency_conversion($shipingPrice,$finbasecurrency,'cad');
								}
								elseif($base_currency != 'USD' && $resultshipping['@attributes']['currency'] == 'USD')
								{
									$finbasecurrency = 'USD'.$base_currency;
									$finalconprice=$this->currency_conversion($shipingPrice,$finbasecurrency,'usd');
								}
								else
								{
									$finalconprice = $shipingPrice;
								}
							
								$rate = array(
										'id' => "eShipper_" . $resultshipping['@attributes']['serviceId'],
										'label' => $np_label_shipping,
										'cost' => $finalconprice,
										'calc_tax' => 'per_order'
										);
								$this->add_rate($rate);
							}
 						}
 						else
						{
 							// qutoe error
							$rate = array();
							$this->add_rate($rate);
						}
					}
				}


				function currency_conversion($Myprice,$to,$baserate)
				{
					if(get_option('exchangeRateusd') && get_option('exchangeRatecad') )
					{	
						if($baserate == 'usd')
						{
							$diffrentConversionRate=get_option('exchangeRateusd');
						}
						else
						{
							$diffrentConversionRate=get_option('exchangeRatecad');
						}
						
						$newConvertedCurrency=($diffrentConversionRate[$to] * $Myprice);
						return round($newConvertedCurrency,2,PHP_ROUND_HALF_UP);
						 
					}
					else
					{
						every_tweleve_hrs_event_func();
						if($baserate == 'usd')
						{
							$diffrentConversionRate=get_option('exchangeRateusd');
						}
						else
						{
							$diffrentConversionRate=get_option('exchangeRatecad');
						}
						$newConvertedCurrency=($diffrentConversionRate[$to] * $Myprice);
						return round($newConvertedCurrency,2,PHP_ROUND_HALF_UP);
					}
					
				}
				
				function sort_rates($a, $b)
				{
					if ($a->cost == $b->cost)
					{
						return 0;
					}

					return ($a->cost < $b->cost) ? -1 : 1;
				}

				function admin_options()
				{
					global $woocommerce;
					wp_nonce_field(plugin_basename(__FILE__) , 'eShipper_options_noncename');					?>
<div class="notice notice-success">
       <p><b>*Note:</b> You must add weight, height, length and width to your products to get Shipping rates from EShipper. For instructions Here is the link: <a href="<?php
					get_home_url(); ?>/wp-content/plugins/woocommerce-eShipper/instructions/index.html" target="_blank">Instruction</a></p>
    </div>
                    <h3><?php
					_e('eShipper', 'woocommerce-eShipper'); ?></h3>

                     <?php 

                             global $error_msg;
                             if($error_msg == 'error'){
                               echo"<h4 style='color: red;'>Wrong EShipper API Settings.Please enter right details</h4>";
                               }       
                     ?>
                    <table class="form-table wooeShipper">
                        <tr valign="top">
                            <th scope="row" class="titledesc"><?php
					_e('Enable/Disable', 'woocommerce-eShipper') ?></th>
                            <td class="forminp">
                                <fieldset>
                                    <legend class="screen-reader-text"><span><?php
					_e('Enable/Disable', 'woocommerce-eShipper') ?></span></legend>
                                    <label for="woocommerce_eShipper_enabled">
                                        <input name="woocommerce_eShipper_enabled" id="woocommerce_eShipper_enabled" type="checkbox" value="1" <?php
					checked($this->options->enabled == 'yes'); ?> />
                                        <?php
					_e('Enable eShipper Webservice', 'woocommerce-eShipper') ?>
                                    </label><br />
                                </fieldset>
                            </td>
                        </tr>
        
         
						<tr valign="top">
                            <th scope="row" class="titledesc"></th>
                            <td class="forminp">
                                <fieldset>
                                    <legend class="screen-reader-text"><span><?php
					_e('Enable/Disable', 'woocommerce-eShipper') ?></span></legend>
                                    <label for="courier_rates_enabled">
                                        <input name="courier_rates_enabled" id="courier_rates_enabled" type="checkbox" value="1" <?php
					checked($this->options->courier == 'yes'); ?> />
                                        <?php
					_e('Courier Rates', 'woocommerce-eShipper') ?>
                                    </label><br />
                                </fieldset>
                            </td>
						</tr>
						<tr valign="top">
                            <th scope="row" class="titledesc"></th>
                            <td class="forminp">
                                <fieldset>
                                    <legend class="screen-reader-text"><span><?php
					_e('Enable/Disable', 'woocommerce-eShipper') ?></span></legend>
                                    <label for="ltl_rates_enabled">
                                    <input name="ltl_rates_enabled" id="ltl_rates_enabled" type="checkbox" value="1" <?php
					checked($this->options->ltl == 'yes'); ?> />
                    <?php
					_e('LTL(Trucking) Rates ', 'woocommerce-eShipper') ?>
                                    </label><br />
                                </fieldset>
                          </td>
						</tr>


						<tr valign="top">
                            <th scope="row" class="titledesc"></th>
							<td class="forminp">
                                <fieldset>
                                    <legend class="screen-reader-text"><span><?php
					_e('Enable/Disable', 'woocommerce-eShipper') ?></span></legend>
                                    <label for="residential_delivery_enabled">
                                    <input name="residential_delivery_enabled" id="residential_delivery_enabled" type="checkbox" value="1" <?php
					checked($this->options->residential == 'yes'); ?> />
                    <?php
					_e('Rate as Residential Shipment', 'woocommerce-eShipper') ?>
                                    </label><br />
                                </fieldset>
							</td>
						</tr>
						<tr valign="top">
                            <th scope="row" class="titledesc"></th>
							<td class="forminp">
                                <fieldset>
                                    <legend class="screen-reader-text"><span><?php
					_e('Enable/Disable', 'woocommerce-eShipper') ?></span></legend>
                                    <label for="signature_enabled">
                                    <input name="signature_enabled" id="signature_enabled" type="checkbox" value="1" <?php
					checked($this->options->signature == 'yes'); ?> />
                    <?php
					_e('Signature Required', 'woocommerce-eShipper') ?>
                                    </label><br />
                                </fieldset>
                          </td>
						</tr>
				 
				 <tr valign="top">
                    <th scope="row" class="titledesc"> </th>
                        <td class="forminp">
                            <fieldset>
                                <legend class="screen-reader-text"><span><?php
					_e('Enable/Disable', 'woocommerce-eShipper') ?></span></legend>
                                    <label for="cheapest_price_only">
                                    <input name="cheapest_price_only" id="cheapest_price_only" type="checkbox" value="1" <?php
					checked($this->options->cheapest == 'yes'); ?> />
                    <?php
					_e('Show Cheapest Option only', 'woocommerce-eShipper') ?>
                                    </label><br />
                                </fieldset>
                          </td>
						  </tr>
						  <tr valign="top">
                    <th scope="row" class="titledesc"> </th>
						  <td class="forminp">
                            <fieldset>
                                <legend class="screen-reader-text"><span><?php
					_e('Enable/Disable', 'woocommerce-eShipper') ?></span></legend>
                                    <label for="boxing_enabled">
                                    <input name="boxing_enabled" id="boxing_enabled" type="checkbox" value="1" <?php
					checked($this->options->boxing_enabled == 'yes'); ?> />
                    <?php
					_e('Enable 4D-Boxing', 'woocommerce-eShipper') ?>
                                    </label><br />
                                </fieldset>
                          </td>
                 </tr>


                        <tr valign="top">
                            <th scope="row" class="titledesc"><?php
					_e('eShipper API Settings', 'woocommerce-eShipper') ?></th>
                            <td class="forminp">
                                <table class="eShipperinnetbl">
                                    <tr>
                                        <td><span class="description"><?php
					_e('API URL *', 'woocommerce-eShipper') ?></span></td>
                                        <td><input type="text" required="required" name="woocommerce_eShipper_api_url" id="woocommerce_eShipper_api_url" style="min-width:200px;" value="<?php
					echo esc_attr($this->options->api_url); ?>" /></td>
                                    </tr>
                                    <tr>
                                        <td><span class="description"><?php
					_e('Username *', 'woocommerce-eShipper') ?></span></td>
                                        <td><input type="text" required="required" name="woocommerce_eShipper_api_user" id="woocommerce_eShipper_api_user" style="min-width:200px;" value="<?php
					echo esc_attr($this->options->api_user); ?>"/></td>
                                    </tr>
                                    <tr>
                                        <td><span class="description"><?php
					_e('Password *', 'woocommerce-eShipper') ?></span></td>
                                        <td><input type="password" required="required" name="woocommerce_eShipper_api_pass" id="woocommerce_eShipper_api_pass" style="min-width:200px;" value="<?php
					echo esc_attr($this->options->api_pass); ?>"/></td>
                                    </tr>
                                </table>
                            </td>
                        </tr>
                        <tr valign="top">
                            <th scope="row" class="titledesc"><?php
					_e('Origin Address Details', 'woocommerce-eShipper') ?></th>
                            <td class="forminp">
                                <table class="eShipperinnetbl">
                                    <tr>
                                        <td>
                                            <span class="description"><?php
					_e('Postal Code *', 'woocommerce-eShipper') ?></span>
                                            <input type="text" required="required" name="woocommerce_eShipper_source_postalcode" id="woocommerce_eShipper_source_postalcode" style="min-width:50px;" value="<?php
					echo esc_attr($this->options->source_postalcode); ?>"/>
                                        </td>
                                        <td>
                                            <span class="description"><?php
					_e('Country *', 'woocommerce-eShipper') ?></span>
                                            <select required="required" name="woocommerce_eShipper_source_country" id="woocommerce_eShipper_source_country">
                                                <option value="">--Select Country--</option>
                                                <?php
					global $wpdb;
					$sql = "SELECT country_name,country_code_char2 FROM wpnp_countries";
					$all_countries = $wpdb->get_results($sql, ARRAY_A);
					foreach($all_countries as $single_country)
						{
						if (esc_attr($this->options->source_country) == $single_country['country_code_char2'])
							{
							$sltctnv = "selected='selected'";
							}
						  else
							{
							$sltctnv = "";
							}

						echo "<option value='" . $single_country['country_code_char2'] . "' " . $sltctnv . ">" . $single_country['country_name'] . "</option>";
						}

?>
                                            </select>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td>
                                            <span class="description"><?php
					_e('State/Province *', 'woocommerce-eShipper') ?></span>
                                            <select required="required" name="woocommerce_eShipper_source_province" id="woocommerce_eShipper_source_province">
                                                <option value="">--Select--</option>
                                            </select>
                                        </td>

                                        <td>
                                            <span class="description"><?php
					_e('Source City *', 'woocommerce-eShipper') ?></span>
                                            <input type="text" name="woocommerce_eShipper_source_city" id="woocommerce_eShipper_source_city" style="min-width:50px;" value="<?php
					echo esc_attr($this->options->source_city); ?>"/>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td>
                                            <span class="description"><?php
					_e('Shipper Name', 'woocommerce-eShipper') ?></span>
                                            <input type="text" name="woocommerce_eShipper_source_name" id="woocommerce_eShipper_source_name" style="min-width:50px;" value="<?php
					echo esc_attr($this->options->source_name); ?>"/>
                                        </td>
                                        <td>
                                            <span class="description"><?php
					_e('Address line 1', 'woocommerce-eShipper') ?></span>
                                            <input type="text" name="woocommerce_eShipper_source_address1" id="woocommerce_eShipper_source_address1" style="min-width:50px;" value="<?php
					echo esc_attr($this->options->source_address1); ?>"/>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td>
                                            <span class="description"><?php
					_e('Address Line 2', 'woocommerce-eShipper') ?></span>
                                            <input type="text" name="woocommerce_eShipper_source_address2" id="woocommerce_eShipper_source_address2" style="min-width:50px;" value="<?php
					echo esc_attr($this->options->source_address2); ?>"/>
                                        </td>
                                        <td>
                                            <span class="description"><?php
					_e('Address line 3', 'woocommerce-eShipper') ?></span>
                                            <input type="text" name="woocommerce_eShipper_source_address3" id="woocommerce_eShipper_source_address3" style="min-width:50px;" value="<?php
					echo esc_attr($this->options->source_address3); ?>"/></td>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td>
                                            <span class="description"><?php
					_e('Phone Number', 'woocommerce-eShipper') ?></span>
                                            <input type="text" name="woocommerce_eShipper_source_phone" id="woocommerce_eShipper_source_phone" style="min-width:50px;" value="<?php
					echo esc_attr($this->options->source_phone); ?>"/>
                                        </td>
                                        <td>
                                            <span class="description"><?php
					_e('Fax Number', 'woocommerce-eShipper') ?></span>
                                            <input type="text" name="woocommerce_eShipper_source_fax" id="woocommerce_eShipper_source_fax" style="min-width:50px;" value="<?php
					echo esc_attr($this->options->source_fax); ?>"/>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td>
                                            <span class="description"><?php
					_e('Type of address', 'woocommerce-eShipper') ?></span>
                                            <input type="text" name="woocommerce_eShipper_source_address_type" id="woocommerce_eShipper_source_address_type" style="min-width:50px;" value="<?php
					echo esc_attr($this->options->source_address_type); ?>"/>
                                        </td>
                                        <td>
                                            <span class="description"><?php
					_e('Company Name', 'woocommerce-eShipper') ?></span>
                                            <input type="text" name="woocommerce_eShipper_source_company_name" id="woocommerce_eShipper_source_company_name" style="min-width:50px;" value="<?php
					echo esc_attr($this->options->source_company_name); ?>"/>
                                        </td>
                                    </tr>
                                </table>
                            </td>
                        </tr>

                           <tr valign="top">
                            <th><b><?php
					_e('Shipping Boxes', 'woocommerce-eShipper') ?></b></th>
                            <td class="boxes-main">
                                <table class="eShipper-boxes">
                                    <tr class="boxes-header-row">
                                    	<th></th>
                                        <th>Height (in)</th>
                                        <th>Width (in)</th>
                                        <th>Length (in)</th>
                                        <th>Box Weight (lbs)</th>
                                        <th>Max Weight (lbs)</th>
                                    </tr>

                                    <?php if(!empty($this->options->boxes)){
                                    	$boxesaary = $this->options->boxes;
                                    	foreach ($boxesaary as $key => $boxvalue) { 
            echo '<tr class="boxes-row">
        	<td class="chk-td"><input  type="checkbox" value="" name="box_chk[]" class="chk-boxes" /></td>
            <td><input type="text" class="height-input" name="woocommerce_eShipper_boxes_height[]" value="'.$boxvalue[0].'"></td>
            <td><input type="text" class="width-input" name="woocommerce_eShipper_boxes_width[]" value="'.$boxvalue[1].'"></td>
            <td><input type="text" class="length-input" name="woocommerce_eShipper_boxes_length[]" value="'.$boxvalue[2].'"></td>
            <td><input type="text" class="length-input" name="woocommerce_eShipper_boxes_box_weight[]" value="'.$boxvalue[3].'"></td>
            <td><input type="text" class="length-input" name="woocommerce_eShipper_boxes_max_weight[]" value="'.$boxvalue[4].'"></td>
            </tr>';
                                    	}
                                    } else { 

                                echo '<tr class="boxes-row">
                            	<td class="chk-td"><input  type="checkbox" value="" name="box_chk[]" class="chk-boxes" /></td>
                                <td><input type="text" class="height-input" name="woocommerce_eShipper_boxes_height[]" value=""></td>
                                <td><input type="text" class="width-input" name="woocommerce_eShipper_boxes_width[]" value=""></td>
                                <td><input type="text" class="length-input" name="woocommerce_eShipper_boxes_length[]" value=""></td>
                                <td><input type="text" class="length-input" name="woocommerce_eShipper_boxes_box_weight[]" value=""></td>
            					<td><input type="text" class="length-input" name="woocommerce_eShipper_boxes_max_weight[]" value=""></td>

                                    </tr>';
                                   } ?>
                                    <tr class="last-box-row">
                                    	<td><input type="button" name="add_box" class="add_box" value="Add Box" /></td>
                                    	<td><input type="button" name="removed_box" class="removed_box" value="Remove Checked Box(s)" /></td>
                                    </tr>
                                </table>
                            </td>
                        </tr>


                        <tr valign="top">
                            <th><b><?php
					_e('Increase all rates by(%)', 'woocommerce-eShipper') ?></b></th>
                            <td>
                                <input type="number" name="woocommerce_eShipper_increase_rate" id="woocommerce_eShipper_increase_rate" style="max-width:50px;" value="<?php
					echo esc_attr($this->options->increase_rate); ?>"/>
                                <span class="description"><?php
					_e('Markup option by percentage factor', 'woocommerce-eShipper') ?></span>
                            </td>
                        </tr>
                    </table>

                    <script>
                    	jQuery(document).ready(function(){

                    		jQuery(document).on('click','.add_box',function(){

                    			//jQuery('<tr class="boxes-row"><td class="chk-td"><input  type="checkbox" value="" name="box_chk[]" class="chk-boxes" /></td><td><input type="text" class="height-input" name="boxes_height[]" value=""></td><td><input type="text" class="width-input" name="boxes_width[]" value=""></td><td><input type="text" class="length-input" name="boxes_length[]" value=""></td></tr>').insertBefore('tr.last-box-row');

                    		
                    		jQuery('table .eShipper-boxes tr:nth-child(2)').clone().insertBefore('tr.last-box-row');	

                    		});

                    		jQuery(document).on('click','.removed_box',function(){
                    		 var numBoxes = jQuery('table .eShipper-boxes tr.boxes-row').length;
                    		 var checkedchkbox = jQuery('table .eShipper-boxes tr.boxes-row input[name="box_chk[]"]:checked').length;

                    		if(numBoxes > 1 && numBoxes > checkedchkbox){
                    			if(checkedchkbox){
                    		jQuery('table.eShipper-boxes tr.boxes-row .chk-boxes').each(function(){	
                    			
                    			 if (jQuery(this).is(":checked")) {
                    				 	
                    			 	jQuery(this).closest('tr').remove();
                    			 }
                    		});
                    			}
                    			else{
                    				alert('Please checked the checkbox.');
                    				}
                    			}
                    			else{
                    				alert('Minimum one row will contains.');
                    			}
                    		});

                    		
                    	});
                    </script>

                    <?php
					}

				function process_admin_options()
					{

					// security check

					if (!isset($_POST['eShipper_options_noncename']) || !wp_verify_nonce($_POST['eShipper_options_noncename'], plugin_basename(__FILE__)))
						{
						return;
						}

					if (isset($_POST['woocommerce_eShipper_enabled'])) $this->options->enabled = 'yes';
					  else $this->options->enabled = 'no';
                  
                     if (isset($_POST['courier_rates_enabled'])) $this->options->courier = 'yes';
					  else $this->options->courier = 'no';

					  if (isset($_POST['ltl_rates_enabled'])) $this->options->ltl = 'yes';
					  else $this->options->ltl = 'no';

					   if (isset($_POST['residential_delivery_enabled'])) $this->options->residential = 'yes';
					  else $this->options->residential = 'no';
					  
					  if (isset($_POST['signature_enabled'])) $this->options->signature = 'yes';
					  else $this->options->signature = 'no';
                  
					   if (isset($_POST['cheapest_price_only'])) $this->options->cheapest = 'yes';
					  else $this->options->cheapest = 'no';
					  
					  if (isset($_POST['boxing_enabled'])) $this->options->boxing_enabled = 'yes';
					  else $this->options->boxing_enabled = 'no';
                 	
                   if(isset($_POST['woocommerce_eShipper_boxes_height'])){
                   	$newboxesarray = array();
                   	$inputboxesheight = wc_clean($_POST['woocommerce_eShipper_boxes_height']);
                   	foreach($inputboxesheight as $key => $box_height){
                   		$box_width= $_POST['woocommerce_eShipper_boxes_width'][$key];
                   		$box_length= $_POST['woocommerce_eShipper_boxes_length'][$key];
                   		$box_weight= $_POST['woocommerce_eShipper_boxes_box_weight'][$key];
                   		$box_max_weight= $_POST['woocommerce_eShipper_boxes_max_weight'][$key];
                   		
                   		$newboxesarray[] = array($box_height,$box_width,$box_length,$box_weight,$box_max_weight);
                   	}

                   	$this->options->boxes = $newboxesarray;

                   }
                   
               		if (isset($_POST['woocommerce_eShipper_api_url'])) $this->options->api_url = wc_clean($_POST['woocommerce_eShipper_api_url']);
					if (isset($_POST['woocommerce_eShipper_api_user'])) $this->options->api_user = wc_clean($_POST['woocommerce_eShipper_api_user']);
					if (isset($_POST['woocommerce_eShipper_api_pass'])) $this->options->api_pass = wc_clean($_POST['woocommerce_eShipper_api_pass']);
					if (isset($_POST['woocommerce_eShipper_source_postalcode'])) $this->options->source_postalcode = wc_clean($_POST['woocommerce_eShipper_source_postalcode']);
					if (isset($_POST['woocommerce_eShipper_source_country'])) $this->options->source_country = wc_clean($_POST['woocommerce_eShipper_source_country']);
					if (isset($_POST['woocommerce_eShipper_source_province'])) $this->options->source_province = wc_clean($_POST['woocommerce_eShipper_source_province']);
					if (isset($_POST['woocommerce_eShipper_source_city'])) $this->options->source_city = wc_clean($_POST['woocommerce_eShipper_source_city']);
					if (isset($_POST['woocommerce_eShipper_source_name'])) $this->options->source_name = wc_clean($_POST['woocommerce_eShipper_source_name']);
					if (isset($_POST['woocommerce_eShipper_source_address1'])) $this->options->source_address1 = wc_clean($_POST['woocommerce_eShipper_source_address1']);
					if (isset($_POST['woocommerce_eShipper_source_address2'])) $this->options->source_address2 = wc_clean($_POST['woocommerce_eShipper_source_address2']);
					if (isset($_POST['woocommerce_eShipper_source_address3'])) $this->options->source_address3 = wc_clean($_POST['woocommerce_eShipper_source_address3']);
					if (isset($_POST['woocommerce_eShipper_source_phone'])) $this->options->source_phone = wc_clean($_POST['woocommerce_eShipper_source_phone']);
					if (isset($_POST['woocommerce_eShipper_source_fax'])) $this->options->source_fax = wc_clean($_POST['woocommerce_eShipper_source_fax']);
					if (isset($_POST['woocommerce_eShipper_source_address_type'])) $this->options->source_address_type = wc_clean($_POST['woocommerce_eShipper_source_address_type']);
					if (isset($_POST['woocommerce_eShipper_source_company_name'])) $this->options->source_company_name = wc_clean($_POST['woocommerce_eShipper_source_company_name']);
					if (isset($_POST['woocommerce_eShipper_increase_rate']) && is_numeric($_POST['woocommerce_eShipper_increase_rate'])) $this->options->increase_rate = wc_clean($_POST['woocommerce_eShipper_increase_rate']);
					  else $this->options->increase_rate = 0;
					if (isset($_POST['woocommerce_eShipper_max_package_size'])) $this->options->max_package_size = wc_clean($_POST['woocommerce_eShipper_max_package_size']);
					if (isset($_POST['woocommerce_eShipper_package_length'])) $this->options->package_length = wc_clean($_POST['woocommerce_eShipper_package_length']);
					if (isset($_POST['woocommerce_eShipper_package_width'])) $this->options->package_width = wc_clean($_POST['woocommerce_eShipper_package_width']);
					if (isset($_POST['woocommerce_eShipper_package_hight'])) $this->options->package_hight = wc_clean($_POST['woocommerce_eShipper_package_hight']);
					if (isset($_POST['woocommerce_eShipper_packaging'])) $this->options->packaging = 'yes';
					  else $this->options->packaging = 'no';
					if (isset($_POST['woocommerce_eShipper_packaging_type'])) $this->options->packaging_type = wc_clean($_POST['woocommerce_eShipper_packaging_type']);

					// update options

					       $admin_userneme = esc_attr($this->options->api_user);
					       $admin_pwd = esc_attr($this->options->api_pass);

					        $data_option_arr = get_option( 'woocommerce_eShipper' );
					       		
					       if(!empty($data_option_arr)){
					        
                            $option_username = $data_option_arr->api_user;
                            $option_userpwd = $data_option_arr->api_pass;
                            $data_option_arr->user_api_key;
                       		 }	
                       		 else{
                        		$option_username = '';
                            	$option_userpwd = '';
                        		}
                      if(!(($admin_userneme == $option_username) && ($admin_pwd == $option_userpwd)))
                       {        
                       $admin_api_url = $this->options->api_url;
                       $admin_api_usrname = $this->options->api_user;
                       $admin_api_userpwd = $this->options->api_pass;

                       $API_KEY ='';
                       $xmldata = '<?xml version="1.0" encoding="UTF-8"?><EShipper  xmlns="http://www.eshipper.net/XMLSchema" username="'.$admin_api_usrname.'" password="'.$admin_api_userpwd.'" version="3.1.9">
                          <UserAuthenticationRequest>
                          </UserAuthenticationRequest>
                          </EShipper>';
					$ch = curl_init();
					curl_setopt($ch, CURLOPT_URL, $admin_api_url);
					curl_setopt($ch, CURLOPT_POST, true);
					curl_setopt($ch, CURLOPT_HTTPHEADER, array(
						'Content-Type: appplication/xml'
					));
					curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
					curl_setopt($ch, CURLOPT_POSTFIELDS, $xmldata);
					$result = curl_exec($ch);
				    $xml = simplexml_load_string($result);
					$json  = json_encode($xml);
 					$configData = json_decode($json, true);
 					
 					if(isset($configData['ErrorReply'])){
 					
 						$this->options->enabled = 'no';
 						$this->options->api_user = $option_username;                            
                        $this->options->api_pass = $option_userpwd;
                        global $error_msg;
                        $error_msg = 'error';	
                        $errorflag = 1;
 					}
 					elseif(isset($configData['UserAuthenticationReply']['AuthenticationData'])){

 						$apiresult = $configData['UserAuthenticationReply']['AuthenticationData'];
 						$API_KEY = $apiresult['@attributes']['apiKey'];
                        $this->options->user_api_key = wc_clean($API_KEY);
                     
 					}
 					
                   
                        }
        			
        			   update_option('woocommerce_eShipper', $this->options);
        					
					}
				}
			}
		}

	add_action('woocommerce_shipping_init', 'eShipper_init');
	function add_eShipper($methods)
		{
		$methods['WC_eShipper'] = 'WC_eShipper';
		return $methods;
		}

	add_filter('woocommerce_shipping_methods', 'add_eShipper');
	}


add_filter( 'cron_schedules', 'isa_add_every_tweleve_hrs' );
function isa_add_every_tweleve_hrs( $schedules ) {
    $schedules['every_tweleve_hrs'] = array(
            'interval'  => 43200,
            'display'   => __( 'Every 12 hrs', 'textdomain' )
    );
    return $schedules;
}

// Schedule an action if it's not already scheduled
if ( ! wp_next_scheduled( 'isa_add_every_tweleve_hrs' ) ) {
    wp_schedule_event( time(), 'every_three_minutes', 'isa_add_every_tweleve_hrs' );
}

// Hook into that action that'll fire every three minutes
add_action( 'isa_add_every_tweleve_hrs', 'every_tweleve_hrs_event_func' );
function every_tweleve_hrs_event_func() 
{

	$ch1 = curl_init();
	curl_setopt($ch1, CURLOPT_URL, 'https://apilayer.net/api/live?access_key=77accc78646c32317d3018eeaf0bfc55&source=cad');
	curl_setopt($ch1, CURLOPT_HEADER, false);
	curl_setopt($ch1, CURLOPT_POST, true);
	curl_setopt($ch1, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch1, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
	curl_setopt($ch1, CURLOPT_SSL_VERIFYPEER, true);

	$response 	= curl_exec($ch1);
	$errno 		= curl_errno($ch1);
	$error 		= curl_error($ch1);

	curl_close($ch1);

	$response 	= json_decode($response, true);
	if($response['success'] == '1'){
	update_option('exchangeRatecad',$response['quotes']);	
	}
	else{

		$exchangeValue = '';
	}

	// usd base 

	$ch2 = curl_init();
	curl_setopt($ch2, CURLOPT_URL, 'https://apilayer.net/api/live?access_key=77accc78646c32317d3018eeaf0bfc55&source=usd');
	curl_setopt($ch2, CURLOPT_HEADER, false);
	curl_setopt($ch2, CURLOPT_POST, true);
	curl_setopt($ch2, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch2, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
	curl_setopt($ch2, CURLOPT_SSL_VERIFYPEER, true);

	$responseusd 	= curl_exec($ch2);
	$errno 		= curl_errno($ch2);
	$error 		= curl_error($ch2);

	curl_close($ch2);

	$responseusd 	= json_decode($responseusd, true);
	if($responseusd['success'] == '1'){
	update_option('exchangeRateusd',$responseusd['quotes']);
	}
	else{

		$exchangeValue = '';
	}
	

	
}